   /*This file is part of KnowMore.org Firefox Extension.

    KnowMore.org Firefox Extension is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License 
    as published by the Free Software Foundation, either version 3 of the 
    License, or (at your option) any later version.

    KnowMore.org Firefox Extension is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KnowMore.org Firefox Extension.  If not, see <http://www.gnu.org/licenses/>.*/

// namespace
this.KnowMore = this.KnowMore || {}

/**
 * Engines
 *
 * XXX document engine interface
 */
KnowMore.Engines = {};

/**
 * Add an engine
 */
KnowMore.addEngine = function(aId, aEngine) {
  this.Engines[aId] = aEngine;
}

/**
 * Parses a URL to be used for searching against local storage, sometimes
 * needed when original urls don't actually point to proper page, i.e., Yahoo
 */

KnowMore.Engines.parseURL = function(aLink, aRegEx, aStr) {
    if (!aLink)
        return null;
    
    //if parsing variables are not set, assumed to be normal link, and sends it back
    if (!aRegEx || !aStr) 
        return KnowMore.Utils.getLinkDomain(aLink); 
    
    var aUrl = aLink.getAttribute("href");
    return aUrl.replace(aRegEx, aStr);
}

/**
 * Utils used by all modules
 */
KnowMore.Utils = {};

/**
* inserts an element directly after another element
*/
KnowMore.Utils.insertAfter = function(eleToInsert, ele){
    if (ele.nextSibling){
        ele.parentNode.insertBefore(eleToInsert, ele.nextSibling);
    }
    else {
        ele.parentNode.appendChild(eleToInsert);
    }
}

/**
* return the domain of a link.
*/
KnowMore.Utils.getLinkDomain = function(link){

    //beware of non-http links as trying to access link.location.hostname can throw an error
    //eg link = <a href="javascript:;"></a>
    // var foo = link.location;     //all good
    // var baa = link.location.hostname;    //throws error
    try {
        return (link && link.hostname) ? link.hostname : null;
    }catch(e){
        return null;
    }
}

/**
* return a profile based on a company name
*/
KnowMore.Utils.getProfileForCmp = function(aCmp) {
    if(!aCmp) 
        return null;

    var profile = KnowMore.DbService.getProfileByCmp(aCmp);
    if(!profile)
        return null;

    profile = this.getProperIconNames(profile);

    return profile;
}

/**
* return a profile based on domain/url
*/
KnowMore.Utils.getProfileForDomain = function(domain, disable){
    //sometimes is domain null
    if(!domain) return null;
    
    //only work with basedomains
    var basedomain = this.getBaseDomain(domain);
    var profile = KnowMore.DbService.getProfileByURL(basedomain, disable);
    if (!profile)
      return null;

    profile = this.getProperIconNames(profile);
    
    //Adds current domain to object for disabling if needed
    profile.cmp_url = basedomain;

    return profile;
}

/**
 * Disables a url by a particular domain/url, return boolean
 */
KnowMore.Utils.disableProfileUrl = function(aUrl) {
    if (KnowMore.DbService.disableProfileUrl(aUrl)) 
        return true;
    
    return false;
}

/**
 * Gets proper names for a particular company's rating icons
 */
KnowMore.Utils.getProperIconNames = function(profile) {
    if(!profile)
        return null;

    var iconMap = {
    r_worker :   "worker",
    r_human  :   "human",
    r_politics : "political",
    r_enviro :   "environment",
    r_ethics :   "business"};

    profile.icons = {};
    for (var field in iconMap)
      profile.icons[ iconMap[field] ] = profile[field];

    return profile;
}

/**
* return a domain stripped of all it's subdomains 
*/
KnowMore.Utils.getBaseDomain = function(domain){
    var regex = /[^\.]+(\.(aero|biz|com|coop|info|cat|museum|mobi|name|net|org|pro|gov|edu|mil|int|co|govt))?(\.(arpa|nato|\w{2}))?$/i;
    var m = domain.match(regex);
    //return all domains in lowercase (makes matching easier later)
    return (m) ? m[0].toLowerCase() : null;
}

/**
 * Returns TRUE if exists user value of preference key with given name. 
 * Otherwise returns FALSE.
 * 
 * @param aPrefName {string}
 * @returns {boolean}
 */
KnowMore.Utils.hasPrefUserValue = function(aPrefName) {
  try {
    var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                     getService(Components.interfaces.nsIPrefBranch);
    
    return prefBranch.prefHasUserValue(aPrefName);
  } catch(e) {
    KnowMore.log("KnowMore.Utils.hasPrefUserValue: " + e);
  }
  
  return false;
};

/**
 * Get a value from a pref or a default value if the pref doesn't exist.
 *
 * @param   aPrefName {string}
 * @param   defaultValue {mixed}
 * @returns the pref's value or the default (if it is missing)
 */
KnowMore.Utils.getPref = function(aPrefName, aDefaultValue) {
  try {
    var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                     getService(Components.interfaces.nsIPrefBranch);
    var type = prefBranch.getPrefType(aPrefName);
    switch (type) {
      case prefBranch.PREF_BOOL:
        return prefBranch.getBoolPref(aPrefName);
      case prefBranch.PREF_INT:
        return prefBranch.getIntPref(aPrefName);
      case prefBranch.PREF_STRING:
        return prefBranch.getCharPref(aPrefName);
    }
  }
  catch (ex) {
    KnowMore.log("KnowMore.Utils.getPref: " + ex);
    /* return the default value */
  }
  
  return aDefaultValue;
};

/**
 * Set preference's value
 *
 * @param aPrefName {string}
 * @param aPrefType {string} [bool|int|char]
 * @param aPrefValue {mixed}
 */
KnowMore.Utils.setPref = function(aPrefName, aPrefType, aPrefValue) {
  try {
    var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                     getService(Components.interfaces.nsIPrefBranch);
    switch(aPrefType) {
      case "bool":
        return prefBranch.setBoolPref(aPrefName, aPrefValue);
      case "int":
        return prefBranch.setIntPref(aPrefName, aPrefValue);
      case "char":
        return prefBranch.setCharPref(aPrefName, aPrefValue);
    }
  } catch(e) {
    KnowMore.log("KnowMore.Utils.setPref: " + e);
  }
  
  return null;
};


/**
 * Returns pointer to file located in profile of parent Mozilla application.
 * 
 * @param aFileName {string}
 * @param aDirsArray (array)
 * @returns {object} Returns instance of nsIFile or FALSE
 */
KnowMore.Utils.getProfileFile = function(aFileName, aDirsArray)
{
  var file = Components.classes["@mozilla.org/file/directory_service;1"].
      getService(Components.interfaces.nsIProperties).
      get("ProfD", Components.interfaces.nsIFile);
  
  if(!(file instanceof Components.interfaces.nsIFile)) 
    return false;
  
  for(var dir in aDirsArray)
    file.append(aDirsArray[dir]);
  
  file.append(aFileName);
  
  return file;
};
