   /*This file is part of KnowMore.org Firefox Extension.

    KnowMore.org Firefox Extension is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License 
    as published by the Free Software Foundation, either version 3 of the 
    License, or (at your option) any later version.

    KnowMore.org Firefox Extension is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KnowMore.org Firefox Extension.  If not, see <http://www.gnu.org/licenses/>.*/

KnowMore.addEngine("ask", {
    /**
     * Supports search results integration
     */
    supportsSearch: true,

    /**
     * Supports homepage integration
     */
    supportsHomepage: false,

    /**
     * List of valid homepages for this engine.
     */
    homepageURLs: ["http://www.ask.com"],

    /**
     * Check if a document is a valid homepage for this engine.
     */
    isValidHomepage: function(doc){
        var locationSansParams = doc.location.toString().split("?")[0];
        return this.homepageURLs.indexOf(locationSansParams) != -1;
    },

    /**
    * Return TRUE if a given document is a search results page
    */
    isValidDoc : function(doc){
        //eg: http://www.google.co.nz/search?q=bob
        //eg: http://www.google.com/search?q=bob
        
        if(!doc)
            return false;

        // disallow non-domain URIs (eg: about:config, chrome URIs)
        if (!doc.domain)
            return false;

        var baseDomain = KnowMore.Utils.getBaseDomain(doc.domain);
        return baseDomain && (baseDomain.search(/(^|\b)ask\b/) > -1) &&
               doc.URL.indexOf("/web?");
    },
    
    /**
    * Returns the search results links for a given search results page
    */
    getItems : function(doc){
        var srLinks = [];
        var links = doc.getElementsByTagName("a");
        for (var i=0; i<links.length; i++){
            var link = links[i];
            if (link.className && link.className.match(/(^|\s)L4$/)) {
                srLinks.push(link);
            }
        }
        
        return srLinks;
    },
    
    /**
    * any additional styles that need to be applied to the icons to get them to position correctly on the page.
    * 
    */
    iconStyle : "border: none; margin: 1px 1px 0px 1px;",
        
    /**
    * addtional styles to be applied to the link 
    */
    linkStyle : "border: none; margin-left: 5px;"
});
